@@ -8,7 +8,25 @@ with raw_source as (
 final as (
 
     select
-{%- if adapter_name == 'SnowflakeAdapter' or adapter_name == 'RedshiftAdapter' %}
+{%- if adapter_name == 'SnowflakeAdapter' %}
+{%- for key, cols in nested.items() %}
+  {%- for col in cols %}
+        {{ key }}:{{ '"' + col + '"' }}::{{ cols[col]["type"].lower() }}
+        {%- if cols[col]["type"].lower() == 'number' -%}
+            ({{ cols[col]["numeric_precision"] }},{{ cols[col]["numeric_scale"] }})
+        {%- endif -%}
+        {{- '' }} as {{ cols[col]["id"] }}{% if not loop.last or columns %},{% endif %}
+  {%- endfor %}
+{%- endfor %}
+{%- for col in columns %}
+        {{ '"' + col['name'] + '"' }}::{{ col["type"].lower() }}
+        {%- if col["type"].lower() == 'number' -%}
+            ({{ col["numeric_precision"] }},{{ col["numeric_scale"] }})
+        {%- endif -%}
+        {{- '' }} as {{ col['id'] }}{% if not loop.last %},{% endif %}
+{%- endfor %}
+
+{%- elif adapter_name == 'RedshiftAdapter' %}
 {%- for key, cols in nested.items() %}
   {%- for col in cols %}
         {{ key }}:{{ '"' + col + '"' }}::{{ cols[col]["type"].lower() }} as {{ cols[col]["id"] }}{% if not loop.last or columns %},{% endif %}
@@ -17,6 +35,7 @@ final as (
 {%- for col in columns %}
         {{ '"' + col['name'] + '"' }}::{{ col["type"].lower() }} as {{ col['id'] }}{% if not loop.last %},{% endif %}
 {%- endfor %}
+
 {%- elif adapter_name == 'BigQueryAdapter' %}
 {%- for key, cols in nested.items() %}
   {%- for col in cols %}
@@ -26,6 +45,7 @@ final as (
 {%- for col in columns %}
         cast({{ col['name'] }} as {{ col["type"].lower().replace("varchar", "string") }}) as {{ col['id'] }}{% if not loop.last %},{% endif %}
 {%- endfor %}
+
 {%- endif %}
 
     from raw_source
