@@ -1,4 +1,5 @@
 """Contains yaml related utils which might get used in places."""
+from copy import deepcopy
 from pathlib import Path
 from typing import Any, Dict
 
@@ -33,3 +34,20 @@ def save_yaml(path: Path, data: Dict[str, Any]) -> None:
     """
     with open(path, "w") as outfile:
         yaml.dump(data, outfile)
+
+
+def deep_merge(new_values, default_values):
+    """Merge new values into default values dict, overrding existing values"""
+
+    def merge(source, destination):
+        for key, value in source.items():
+            if isinstance(value, dict):
+                # get node or create one
+                node = destination.setdefault(key, {})
+                merge(value, node)
+            else:
+                destination[key] = value
+        return destination
+
+    default = deepcopy(default_values)
+    return merge(new_values, default)
